/*
 * display.c
 *
 *  Created on: 29.08.2016
 *      Author: wn
 */


#include <msp430g2553.h>
#include <stdint.h>
#include <intrinsics.h>

#include "display.h"
#include "gpio.h"
#include "time.h"


const uint32_t DISPLAY_CYCLE = 5;

const uint16_t INIT_CYCLE_DELAY = 200;

typedef enum {
  ALL_ID = 10,
  H_ID,
  I_ID,
  EMPTY_ID,
} tPatternId;

const tPin ALL_PATTERN[] = { SEG_A, SEG_B, SEG_C, SEG_D, SEG_E, SEG_F, SEG_G, PINS_END };
const tPin H_PATTERN[] = { SEG_B, SEG_C, SEG_E, SEG_F, SEG_G, PINS_END };
const tPin I_PATTERN[] = { SEG_B, SEG_C, PINS_END };
const tPin L_PATTERN[] = { SEG_D, SEG_E, SEG_F, PINS_END };
const tPin O_PATTERN[] = { SEG_A, SEG_B, SEG_C, SEG_D, SEG_E, SEG_F, PINS_END };
const tPin EMPTY_PATTERN[] = { PINS_END };
const tPin NUMBER0[] = { SEG_A, SEG_B, SEG_C, SEG_D, SEG_E, SEG_F, PINS_END };
const tPin NUMBER1[] = { SEG_B, SEG_C, PINS_END };
const tPin NUMBER2[] = { SEG_A, SEG_B, SEG_G, SEG_E, SEG_D, PINS_END };
const tPin NUMBER3[] = { SEG_A, SEG_B, SEG_G, SEG_C, SEG_D, PINS_END };
const tPin NUMBER4[] = { SEG_F, SEG_G, SEG_B, SEG_C, PINS_END };
const tPin NUMBER5[] = { SEG_A, SEG_F, SEG_G, SEG_C, SEG_D, PINS_END };
const tPin NUMBER6[] = { SEG_A, SEG_C, SEG_D, SEG_E, SEG_F, SEG_G, PINS_END };
const tPin NUMBER7[] = { SEG_A, SEG_B, SEG_C, PINS_END };
const tPin NUMBER8[] = { SEG_A, SEG_B, SEG_C, SEG_D, SEG_E, SEG_F, SEG_G, PINS_END };
const tPin NUMBER9[] = { SEG_A, SEG_B, SEG_C, SEG_D, SEG_F, SEG_G, PINS_END };

const tPin *NUMBERS[] = { NUMBER0, NUMBER1, NUMBER2, NUMBER3, NUMBER4,
    NUMBER5, NUMBER6, NUMBER7, NUMBER8, NUMBER9, ALL_PATTERN, H_PATTERN,
    I_PATTERN, EMPTY_PATTERN };

const tPin DIGITS[] = { DIGIT_0, DIGIT_1, DIGIT_2, PINS_END };


uint8_t digitValues[] = { EMPTY_ID, EMPTY_ID, EMPTY_ID };



void displayInit(void *handleArg) {
  for (tPin d = DIGIT_0; d <= DIGIT_2; d++) {
    gpioSetPin(d, LOW);
    for (tPin s = SEG_A; s <= SEG_F; s++) {
      gpioSetPin(s, HIGH);
      ms_active_delay(INIT_CYCLE_DELAY);
      gpioSetPin(s, LOW);
    }
    gpioSetPin(d, HIGH);
  }
  for (tPin d = DIGIT_2; d >= DIGIT_0; d--) {
    gpioSetPin(d, LOW);
    gpioSetPin(SEG_G, HIGH);
    ms_active_delay(INIT_CYCLE_DELAY);
    gpioSetPin(SEG_G, LOW);
    gpioSetPin(d, HIGH);
  }
}

static void showNumber(uint8_t n) {
  for (const tPin *pattern = NUMBERS[ALL_ID]; *pattern != PINS_END; pattern++) {
    gpioSetPin(*pattern, LOW);
  }
  for (const tPin *pattern = NUMBERS[n]; *pattern != PINS_END; pattern++) {
    gpioSetPin(*pattern, HIGH);
  }
}

void displaySetValue(uint8_t v) {
  uint32_t a,b,c,d;
  a = v;
  b = a / 100;
  c = (a - b * 100) / 10;
  d = a - c * 10 - b * 100;
  digitValues[2] = (b == 0) ? EMPTY_ID : (uint8_t)b;
  digitValues[1] = ((c == 0) && (b == 0)) ? EMPTY_ID : (uint8_t)c;
  digitValues[0] = (uint8_t)d;
}

void displayExec(void *handleArg) {
  static uint8_t activeDigit = 0;

  activeDigit++;
  if (activeDigit == 3) {
    activeDigit = 0;
  }

  for (const tPin *digit = DIGITS; *digit != PINS_END; digit++) {
    gpioSetPin(*digit, HIGH);
  }

  gpioSetPin(DIGITS[activeDigit], LOW);

  showNumber(digitValues[activeDigit]);
}
